
from flask import Flask
from flask_sqlalchemy import SQLAlchemy
from flask_login import LoginManager, UserMixin, login_user, login_required, logout_user, current_user
import os
from flask_babel import Babel
from dotenv import load_dotenv

load_dotenv()

app = Flask(
    __name__,
    template_folder= os.path.join(os.path.dirname(os.path.abspath(__file__)), 'templates'),
    static_folder= os.path.join(os.path.dirname(os.path.abspath(__file__)), 'static')
)

app.config['BABEL_DEFAULT_LOCALE'] = 'es'
app.config['LANGUAGES'] = [
    'en',
    'es',
    'gl',
]

babel = Babel(app)

postgres_user = os.getenv('POSTGRES_USER')
postgres_password = os.getenv('POSTGRES_PASSWORD')
postgres_host = os.getenv('POSTGRES_HOST')

app.config['SQLALCHEMY_DATABASE_URI'] = f'postgresql://{postgres_user}:{postgres_password}@{postgres_host}:5432/notlar'
app.config['SECRET_KEY'] = 'your_secret_key_here'

db = SQLAlchemy(app)

login_manager = LoginManager(app)
login_manager.login_view = '/'
login_manager.init_app(app)

UPLOAD_FOLDER = os.path.join(app.root_path, 'static', 'img', 'users')

if not os.access(UPLOAD_FOLDER, os.W_OK):
    print("Upload folder does not have write permissions!")

if not os.path.exists(UPLOAD_FOLDER):
    os.makedirs(UPLOAD_FOLDER)

app.config['UPLOAD_FOLDER'] = UPLOAD_FOLDER


from notlar import routes, models, auth




from flask import request, jsonify, redirect, url_for
from werkzeug.security import generate_password_hash, check_password_hash
from notlar import app, db
from notlar.models import User
from sqlalchemy.exc import IntegrityError
import re
from flask_login import login_user, login_required, logout_user, current_user
import os
from werkzeug.utils import secure_filename

# Regex for email validation
EMAIL_PATTERN = r'^[a-zA-Z0-9_.+-]+@[a-zA-Z0-9-]+\.[a-zA-Z0-9-.]+$'

# Reggex for password validation
PASSWORD_LENGTH_PATTERN = r'^.{8,}$'
PASSWORD_SPECIAL_CHARACTER_PATTERN = r'[!@#$%^&*(),.?":{}|<>]'
PASSWORD_NUMBER_PATTERN = r'\d'

# Allowed extensions for picture uploading
ALLOWED_EXTENSIONS = {'png', 'jpg', 'jpeg', 'gif'}

# Default picture for users without a pic configured in the profile
DEFAULT_PIC = 'default.png'


def allowed_file(filename):
    allowed_extensions = {'png', 'jpg', 'jpeg', 'gif'}
    file_extension = filename.rsplit('.', 1)[1].lower() if '.' in filename else None
    return '.' in filename and file_extension in allowed_extensions


@app.route('/register', methods=['POST'])
def register():
    try:
        data = request.get_json()
        email = data.get('email')
        password = data.get('password')
        name = data.get('name')
        last_name = data.get('last_name')
        phone_number = data.get('phone_number')
        telegram_user = data.get('telegram_user')

        if not email or not password or not name or not last_name:
            return jsonify({'message': 'Missing required fields (email, username, password, name, or last_name)'}), 400

        if not re.match(EMAIL_PATTERN, email):
            return jsonify({'message': 'Invalid email format'}), 400

        app.logger.info(f"Received registration request for email: {email}")
        hashed_password = generate_password_hash(password, method='pbkdf2:sha256')
        new_user = User(email=email, username=email, password=hashed_password, name=name, last_name=last_name,
                        phone_number=phone_number, telegram_user=telegram_user)
        db.session.add(new_user)
        db.session.commit()
        return jsonify({'message': 'User registered successfully!'}), 201
    except IntegrityError as e:
        app.logger.warning(f"Registration failed due to IntegrityError: {e}")
        db.session.rollback()
        if 'email' in e.orig.args[0]:
            return jsonify({'message': 'Email already exists'}), 400
        elif 'username' in e.orig.args[0]:
            return jsonify({'message': 'Username already exists'}), 400
        else:
            return jsonify({'message': 'Registration failed due to a database error'}), 500
    except Exception as e:
        app.logger.error(f"Registration failed due to an unexpected error: {e}")
        db.session.rollback()
        return jsonify({'message': 'Registration failed due to an unexpected error'}), 500


@app.route('/login', methods=['GET', 'POST'])
def login():
    try:
        data = request.get_json(force=True)
        email_or_username = data.get('email') or data.get('username')
        password = data.get('password')

        user_db = User.query.filter(
            db.or_(User.email == email_or_username, User.username == email_or_username)
        ).first()

        if not user_db or not check_password_hash(user_db.password, password):
            return jsonify({'message': 'Invalid email/username or password'}), 401

        login_user(user_db)

        return jsonify({'logged': 'successful'}), 200

    except Exception as e:
        app.logger.warning(e)
        return jsonify({'message': 'Internal Server Error. Ask the administrator'}), 500


@app.route('/logout', methods=['GET'])
@login_required
def logout():
    try:
        logout_user()
        return redirect(url_for('index'))
    except Exception as e:
        app.logger.error(e)
        raise


@app.route('/update_profile', methods=['POST'])
@login_required
def update_profile():
    try:
        user = current_user
        data = request.form
        new_name = data.get('name')
        new_last_name = data.get('last_name')
        new_telegram_user = data.get('telegram_user')
        old_password = data.get('old_password')
        new_password = data.get('new_password')
        confirm_password = data.get('confirm_password')

        if old_password and (not new_password or not confirm_password):
            return jsonify({'message': 'New password and confirmation are required when changing the password'}), 400

        if old_password and not check_password_hash(user.password, old_password):
            return jsonify({'message': 'Incorrect old password'}), 400

        if new_password != confirm_password:
            return jsonify({'message': 'New password fields do not match'}), 400

        if new_password and not (
            re.search(PASSWORD_LENGTH_PATTERN, new_password) and
            re.search(PASSWORD_SPECIAL_CHARACTER_PATTERN, new_password) and
            re.search(PASSWORD_NUMBER_PATTERN, new_password) and
            new_password != old_password
        ):
            return jsonify({
                'message': 'Invalid new password. Password must be at least 8 characters long, '
                           'contain at least one special character, one number, and must not be '
                           'the same as the old password'
            }), 400

        # Update user data
        user.name = new_name if new_name else user.name
        user.last_name = new_last_name if new_last_name else user.last_name
        user.telegram_user = new_telegram_user if new_telegram_user else user.telegram_user

        # Update password if a new one is provided
        if new_password:
            user.password = generate_password_hash(new_password, method='pbkdf2:sha256')

        if 'profile_picture' in request.files:
            file = request.files['profile_picture']
            if file.filename != '' and allowed_file(file.filename):
                filename = secure_filename(file.filename)
                file_path = os.path.join(app.config['UPLOAD_FOLDER'], filename)
                app.logger.warning(file_path)
                file.save(file_path)
                app.logger.warning(file_path)
                user.profile_picture = filename

        db.session.commit()

        # Return the updated user data
        return jsonify({
            'name': user.name,
            'last_name': user.last_name,
            'telegram_user': user.telegram_user,
            'email': user.email,
            'username': user.username
        }), 200
    except Exception as e:
        app.logger.warning(str(e))
        db.session.rollback()
        return jsonify({'message': 'Internal Server Error. Ask the administrator'}), 500



from flask import request, jsonify, render_template
from notlar import app, db
from notlar.models import User, Note
from datetime import datetime
from flask import redirect, url_for
from . import login_manager, current_user
from sqlalchemy import desc


@login_manager.user_loader
def load_user(id):
    return User.query.get(int(id))


@app.errorhandler(500)
@app.errorhandler(400)
def internal_server_error(error):
    return render_template('error.html'), error.code


@app.errorhandler(404)
def not_found(error):
    return render_template('not_found.html'), error.code


@app.route('/', methods=['GET'])
def index():
    if current_user.is_authenticated:
        return redirect(url_for('dashboard_notes_management'))
    return render_template('index.html')


@app.route('/dashboard_notes_management', methods=['GET'])
def dashboard_notes_management():
    if not current_user.is_authenticated:
        return redirect(url_for('/'))
    current_date_time = datetime.now()
    today_formatted_date = current_date_time.strftime('%d/%m/%Y')
    return render_template(
        'dashboard_notes_management.html',
        user_email=current_user.email,
        user_name=current_user.name,
        todays_date=today_formatted_date
    ), 200


@app.route('/dashboard_all_notes', methods=['GET'])
def dashboard_all_notes():
    if not current_user.is_authenticated:
        return redirect(url_for('/'))
    return render_template(
        'dashboard_all_notes.html',
        user_email=current_user.email,
        user_name=current_user.name
    ), 200


@app.route('/dashboard_home', methods=['GET'])
def dashboard_home():
    if not current_user.is_authenticated:
        return redirect(url_for('/'))
    return render_template(
        'dashboard_home.html',
        user_email=current_user.email,
        user_name=current_user.name
    ), 200


@app.route('/contact', methods=['GET'])
def contact():
    return render_template('contact.html')


@app.route('/kanban', methods=['GET'])
def kanban():
    if not current_user.is_authenticated:
        return redirect(url_for('/'))
    return render_template(
        'kanban.html'
    ), 200


@app.route('/settings', methods=['GET'])
def settings():
    if not current_user.is_authenticated:
        return redirect(url_for('/'))
    return render_template('settings.html'), 200


@app.route('/get_notes', methods=['GET'])
def get_notes():
    if not current_user.is_authenticated:
        return jsonify({"error": "User not authenticated"}), 401

    date_str = request.args.get('date')
    selected_date = datetime.strptime(date_str, '%Y-%m-%d').date()
    notes = Note.query.filter_by(user_id=current_user.id, created_at=selected_date).all()

    notes_data = [
        {
            "date": note.created_at.strftime('%Y-%m-%d'),
            "content": note.text,
            "number": note.id,
            "color": note.color
        } for note in notes]
    return jsonify(notes_data), 200


@app.route('/all_notes', methods=['GET'])
def list_notes_by_date_range():
    if not current_user.is_authenticated:
        return jsonify({"error": "User not authenticated"}), 401

    start_date_str = request.args.get('start')
    end_date_str = request.args.get('end')

    start_date = datetime.strptime(start_date_str, '%Y-%m-%d') if start_date_str else None
    end_date = datetime.strptime(end_date_str, '%Y-%m-%d') if end_date_str else None

    query = Note.query.filter_by(user_id=current_user.id)

    if start_date and end_date:
        query = query.filter(Note.created_at >= start_date, Note.created_at <= end_date)
    elif start_date:
        query = query.filter(Note.created_at >= start_date)
    elif end_date:
        query = query.filter(Note.created_at <= end_date)

    query = query.order_by(desc(Note.created_at))

    notes = query.all()

    notes_data = [
        {
            "date": note.created_at.strftime('%Y-%m-%d'),
            "content": note.text,
            "number": note.id,
            "color": note.color
        } for note in notes]

    return jsonify(notes_data), 200


@app.route('/create_note', methods=['POST'])
def create_note():
    if not current_user.is_authenticated:
        return jsonify({"error": "User not authenticated"}), 401

    data = request.json
    title = data.get('title', '')
    text = data.get('text', '')

    # Let's keep like this since we don't handle colors yet
    color = data.get('color', '#FFFFFF')

    created_at = datetime.strptime(data.get('created_at'), '%Y-%m-%d').date()

    new_note = Note(title=title, text=text, color=color, user=current_user, created_at=created_at)
    db.session.add(new_note)
    db.session.commit()

    return jsonify({"message": "Note created successfully"}), 201


@app.route('/delete_note/<int:note_id>', methods=['DELETE'])
def delete_note(note_id):
    if not current_user.is_authenticated:
        return jsonify({"error": "User not authenticated"}), 401

    note = Note.query.get(note_id)

    if note and note.user_id == current_user.id:
        db.session.delete(note)
        db.session.commit()
        return jsonify({"message": "Note deleted successfully"}), 200
    else:
        return jsonify({"error": "Note not found or unauthorized"}), 404



from setuptools import setup, find_packages

with open('requirements.txt') as f:
    requirements = f.read().splitlines()

setup(
    name='notlar',
    version='0.5',
    author='Adrian Fusco',
    packages=find_packages(),
    install_requires=requirements,
    entry_points={
        'console_scripts': [
            'notlar = wsgi:run_gunicorn',
        ],
    },
    classifiers=[
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 3.10',
    ],
)



from notlar import db
from flask_login import UserMixin


class User(db.Model, UserMixin):
    __tablename__ = 'users'
    id = db.Column(db.Integer, primary_key=True)
    email = db.Column(db.String(120), unique=True, nullable=False)
    username = db.Column(db.String(80), unique=True, nullable=False)
    password = db.Column(db.String(255), nullable=False)
    name = db.Column(db.String(80))
    last_name = db.Column(db.String(80))
    phone_number = db.Column(db.String(20))
    telegram_user = db.Column(db.String(80))
    profile_picture = db.Column(db.String(255))
    notes = db.relationship('Note', backref='user', lazy=True)

    # Methods required by flask-login module
    # It will be handled by the login_required decorator
    def get_id(self):
        return str(self.id)

    @property
    def is_active(self):
        return True

    @property
    def is_authenticated(self):
        return True

    @property
    def is_anonymous(self):
        return True


class Note(db.Model):
    __tablename__ = 'notes'
    id = db.Column(db.Integer, primary_key=True)
    title = db.Column(db.String(255), nullable=False)
    text = db.Column(db.String(1000), nullable=False)
    color = db.Column(db.String(7), nullable=False)
    user_id = db.Column(db.Integer, db.ForeignKey('users.id'), nullable=False)
    created_at = db.Column(db.DateTime, default=db.func.current_timestamp())
    updated_at = db.Column(db.DateTime, default=db.func.current_timestamp(), onupdate=db.func.current_timestamp())
